/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.store;

import hu.midori.kosmos.server.util.WebdavUtils;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Date;
import java.util.Enumeration;

import org.apache.commons.httpclient.HttpURL;
import org.apache.commons.httpclient.HttpsURL;
import org.apache.commons.httpclient.URIException;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.webdav.lib.WebdavResource;

/**
 * WebDAV-based store implementation.
 * Writes the cached data to WebDAV resources and uses their public
 * URLs to access them.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class WebdavStaticContentStore implements StaticContentStore { 
    private final static Log log = LogFactory.getLog(WebdavStaticContentStore.class);

    /* Spring-injected properties. */
    private String webdavUrl;
    private String webdavUser;
    private String webdavPassword;
    private String clientUrl;

	public void setWebdavUrl(String webdavUrl) {
		this.webdavUrl = webdavUrl;
	}

	public void setWebdavUser(String webdavUser) {
		this.webdavUser = webdavUser;
	}

	public void setWebdavPassword(String webdavPassword) {
		this.webdavPassword = webdavPassword;
	}
    
    public void setClientUrl(String clientUrl) {
		this.clientUrl = clientUrl;
	}

    /** This method is left empty as there is no optimization opportunity. */
	public void begin() {
	}

    /** This method is left empty as there is no optimization opportunity. */
	public void end() {
	}

	/** 
     * Saves the passed stream to a WebDAV repository-based cache and returns 
     * the absolute URL pointing to the resulted file.
     */
    public String storeFile(String fileName, InputStream in) throws URIException, IOException {
    	// init       
        log.debug(String.format("Connecting to \"%s\" as \"%s\" (\"%s\")...", webdavUrl, webdavUser, webdavPassword));
        if(StringUtils.isBlank(webdavUrl))
            throw new IllegalStateException("'webdavUrl' was not specified as servlet-context init-params");
        
        // determine server-side cache path and cache URL
        boolean isHttps = webdavUrl.startsWith("https://");        
        HttpURL webdavRootUrl = isHttps ? new HttpsURL(webdavUrl) : new HttpURL(webdavUrl);  
        if (!isHttps) {
        	if(!StringUtils.isBlank(webdavUser))
        		webdavRootUrl.setUser(webdavUser);
        	if(!StringUtils.isBlank(webdavPassword))
        		webdavRootUrl.setPassword(webdavPassword);
        }
        // use "webdav.url" if "webdav.clientUrl" was not specified
        if(StringUtils.isBlank(clientUrl))
            clientUrl = webdavRootUrl.getURI();

        // connect
        WebdavResource root = new WebdavResource(webdavRootUrl);
        
        String cacheDirName = "kosmos-cache";
        String cacheDirPath = root.getPath() + "/" + cacheDirName + "/";
        
        String cachedFilename = getCachedFileName(fileName);
        String cachedFilePath = cacheDirPath + cachedFilename;
        String cachedFileUrl = clientUrl + "/" + cacheDirName + "/" + URLEncoder.encode(cachedFilename, "utf-8");
        
        log.info("cacheDirPath: " + cacheDirPath);
        log.info("cachedFilePath: " + cachedFilePath);
        log.info("cachedFileUrl: " + cachedFileUrl);

        // init cache: create WebDAV collection if not existing
        WebdavResource cacheDir = null;
        WebdavUtils.dumpResourceChildren(root);
        cacheDir = root.getChildResources().getResource(cacheDirName);
        if(cacheDir == null) {
            log.info(String.format("Creating WebDAV collection \"%s\"...", cacheDirPath));
        	root.mkcolMethod(cacheDirPath);

            WebdavUtils.dumpResourceChildren(root);        
            cacheDir = root.getChildResources().getResource(cacheDirName);
            if(cacheDir == null)
            	throw new IllegalStateException("Couldn't find WebDAV cache directory");
        }
        
		// clear cache: remove old resource(s) from WebDAV
		String cachedWildcard = getCachedWildcard(fileName);
		Enumeration oldCachedFiles = cacheDir.getChildResources().getResources();
		while(oldCachedFiles.hasMoreElements()) {
			WebdavResource oldCachedFile = (WebdavResource)oldCachedFiles.nextElement();
			if(oldCachedFile.isCollection())
				continue;
			
			if(oldCachedFile.getName().endsWith(cachedWildcard)) {
				log.debug(String.format("Deleting outdated WebDAV resource \"%s\"...", oldCachedFile.getName()));
				root.deleteMethod(oldCachedFile.getPath());
			}
		}

		// fill cache: put new resource to WebDAV
        log.debug(String.format("Putting WebDAV resource \"%s\"...", cachedFilename));        
        root.putMethod(cachedFilePath, in);
        
        cacheDir.close();
        root.close();

        return cachedFileUrl;
    }        
    
    /** 
     * Returns the part of the filename for a cached item without the timestamp
     * prefix. This is used to found the previous versions of the same cache item.
     */
    private static String getCachedWildcard(String filename) throws UnsupportedEncodingException {
    	return encodeFilename("", filename);
    }

    /** 
     * Returns the local filename for a cached item, by adding a timestamp prefix
     * so that browsers will not cache the item if it actually changes.
     */
    private static String getCachedFileName(String filename) throws UnsupportedEncodingException {
        return encodeFilename(Long.toString(new Date().getTime()), filename);            
    }
    
    /** 
     * Encodes the local filename for a cached item.
     * @param filename can contain any character.
     * @return the filename which contain only file-system-compatible characters.
     */
    private static String encodeFilename(String prefix, String filename) throws UnsupportedEncodingException {
    	return URLEncoder.encode(String.format("%s_%s", prefix, filename), "utf-8");
    }
}
