/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.svn;

import hu.midori.kosmos.model.ScmRepositoryChange;
import hu.midori.kosmos.model.SvnRepository;
import hu.midori.kosmos.protocol.SvnService;
import hu.midori.kosmos.server.AbstractKosmosService;
import hu.midori.kosmos.server.AbstractKosmosServiceTest;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Unit test.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class SvnServiceTest extends AbstractKosmosServiceTest {
    private SvnService service = new SvnServiceImpl();

    @Configuration(beforeTestMethod=true)
    @Override
    public void setup() {
        super.setup();

        setupServiceMockEnvironment((AbstractKosmosService)service);
    }

    /** Tests with invalid URL. */
    @Test()
    public void testInvalidUrl() {
        assert service.getRepositories("http://www.nosuchdomain.com/svn").isEmpty();
    }

    /** Tests accessing secured repositories. */
    @Test()
    public void testAuthentication() {
        // test secured without signing in
        assert service.getRepositories("https://svn.labs.jboss.com/trunk/labs/demo").isEmpty();

        // test secured with signing in
        testRepository("https://myusername:mypassword@svn.labs.jboss.com/trunk/labs/demo");
    }

    /** Tests public Apache repositories. */
    @Test()
    public void testApacheRepositories() {
        testRepository("http://svn.apache.org/repos/asf/commons");
        testRepository("http://svn.apache.org/repos/asf/ibatis");
        testRepository("http://svn.apache.org/repos/asf/httpd");
        testRepository("http://svn.apache.org/repos/asf/jackrabbit");
    }

    /** Tests public Codehaus repositories. */
    @Test()
    public void testCodehausRepositories() {
        testRepository("https://svn.codehaus.org/picocontainer");
        testRepository("https://svn.codehaus.org/sitemesh");
        testRepository("https://svn.codehaus.org/xfire");
        testRepository("https://svn.codehaus.org/xstream");
    }

    /** Tests public JBoss repositories. */
    @Test()
    public void testJBossRepositories() {
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/demo");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/jbossweb");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/kosmos");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/reportingservices");
    }

    /** Tests a single repository. */
    @Test()
    protected void testRepository(String url) {
        List<SvnRepository> repositories = service.getRepositories(url);
        assert !repositories.isEmpty();

        // test repositories
        for(SvnRepository repository : repositories) {
            assert !StringUtils.isBlank(repository.getLocation());
            assert repository.getRevision() > 0;

            assert repository.getDateCreated().getTime() > 0;
            assert !repository.getCommits().isEmpty();
            for(ScmRepositoryChange commit : repository.getCommits()) {
                assert commit.getRevision() > 0;
                assert commit.getDate() != null;
                assert !StringUtils.isBlank(commit.getAuthor());
                assert commit.getMessage().length() >= 0; // message can be blank

                assert commit.getAddedFiles() >= 0;
                assert commit.getDeletedFiles() >= 0;
                assert commit.getModifiedFiles() >= 0;
                assert commit.getReplacedFiles() >= 0;
                assert (commit.getAddedFiles() + commit.getDeletedFiles() + commit.getModifiedFiles() + commit.getReplacedFiles()) > 0;
            }

            assert repository.getCommitsTotal() > 0;
            assert (repository.getCommitsToday() >= 0) && (repository.getCommitsToday() <= repository.getCommitsTotal());
            assert (repository.getCommitsLast7Days() >= 0) && (repository.getCommitsLast7Days() <= repository.getCommitsTotal());
            assert (repository.getCommitsLast31Days() >= 0) && (repository.getCommitsLast31Days() <= repository.getCommitsTotal());

            assert repository.getCommittersTotal() > 0;
            assert (repository.getCommittersToday() >= 0) && (repository.getCommittersToday() <= repository.getCommittersTotal());
            assert (repository.getCommittersLast7Days() >= 0) && (repository.getCommittersLast7Days() <= repository.getCommittersTotal());
            assert (repository.getCommittersLast31Days() >= 0) && (repository.getCommittersLast31Days() <= repository.getCommittersTotal());

            AbstractKosmosServiceTest.testZoomableImageLocator(repository.getFilesPerFileTypeChartLocator());
            AbstractKosmosServiceTest.testZoomableImageLocator(repository.getCommitsPerAuthorChartLocator());
            AbstractKosmosServiceTest.testZoomableImageLocator(repository.getCommitsPerFileChartLocator());
            AbstractKosmosServiceTest.testZoomableImageLocator(repository.getCommitsPerWeekChartLocator());
            AbstractKosmosServiceTest.testZoomableImageLocator(repository.getRepoEntriesPerWeekChartLocator());

            assert repository.getDirs() >= 0;
            assert repository.getFiles() >= 0;
            assert repository.getTotalFileSize() >= 0;

            assert (repository.getFiles() == 0) || !repository.getFilesPerFileType().isEmpty();
            assert !repository.getCommitsPerAuthor().isEmpty();
            assert !repository.getCommitsPerFile().isEmpty();

            assert repository.getAge() >= 0;
            assert (repository.getLatestTouchAge() >= 0) && (repository.getLatestTouchAge() <= repository.getAge()) : String.format("Repository should be older or same age (%d) like its latest touch (%d)", repository.getAge(), repository.getLatestTouchAge());
            assert repository.getLatestTouchAgeInDays() >= 0;
            assert (repository.getActivity() >= 0) && (repository.getActivity() <= 1.0) : String.format("Activity should be in [0 ... 1.0], not %d", repository.getActivity());
        }
    }
}
